# Put your header here!

from plane import Vector2, Point2

class Bacterium:
    "Base class for bacteria living in an Environment"
    def __init__(self, env, position):
        "Initialize a bacterium at `position` (a Point2) and add to Environment `env`"
        self.env = env
        self.alive = True
        self.position = position
        if not self.env.is_available(position):
            raise ValueError("Specified position not available")
        self.env.add(self)

    def can_move(self, v):
        "Is it allowable for this bacterium to move by Vector2 `v`?"
        return self.env.is_available(self.position + v)

    def move(self, v):
        "Move by Vector2 `v`"
        self.position += v

    def nutrients_available(self, v=Vector2(0, 0)):
        """
        Return quantity of nutrients available at the bacterium's position.
        Or, if a vector `v` is given, return the quantity available at
        position+v.
        """
        return self.env.get_nutrients(self.position + v)

    def consume(self, k):
        """
        Consume `k` nutrients at the current position (raises ValueError
        if not enough nutrients are available.
        """
        self.env.consume_nutrients(self.position, k)

    def die(self):
        "Make this bacterium die"
        self.alive = False

    def update(self):
        "Advance the simulation one time step (by doing nothing)"
        # Subclasses should override this method to do things!

    def __str__(self):
        "Human-readable string representation used by `print`"
        return "{}(position=({},{}))".format(self.__class__.__name__,self.position.x,self.position.y)

    def __repr__(self):
        "Unambiguous string representation used in the REPL"
        return str(self)


# Add Eater, Floater, Divider below!
